@@ -29,6 +29,11 @@
 
 static connman_bool_t global_offlinemode = FALSE;
 
+connman_bool_t __connman_manager_get_offlinemode(void)
+{
+	return global_offlinemode;
+}
+
 static void append_profiles(DBusMessageIter *dict)
 {
 	DBusMessageIter entry, value, iter;
@@ -125,15 +130,10 @@ static void append_connections(DBusMessageIter *dict)
 	dbus_message_iter_close_container(dict, &entry);
 }
 
-static void append_technologies(gboolean powered, DBusMessageIter *dict)
+static void append_available_technologies(DBusMessageIter *dict)
 {
 	DBusMessageIter entry, value, iter;
-	const char *key;
-
-	if (powered)
-		key = "EnabledTechnologies";
-	else
-		key = "Technologies";
+	const char *key = "AvailableTechnologies";
 
 	dbus_message_iter_open_container(dict, DBUS_TYPE_DICT_ENTRY,
 								NULL, &entry);
@@ -146,7 +146,55 @@ static void append_technologies(gboolean powered, DBusMessageIter *dict)
 
 	dbus_message_iter_open_container(&value, DBUS_TYPE_ARRAY,
 					DBUS_TYPE_STRING_AS_STRING, &iter);
-	__connman_notifier_device_type_list(powered, &iter);
+	__connman_notifier_list_registered(&iter);
+	dbus_message_iter_close_container(&value, &iter);
+
+	dbus_message_iter_close_container(&entry, &value);
+
+	dbus_message_iter_close_container(dict, &entry);
+}
+
+static void append_enabled_technologies(DBusMessageIter *dict)
+{
+	DBusMessageIter entry, value, iter;
+	const char *key = "EnabledTechnologies";
+
+	dbus_message_iter_open_container(dict, DBUS_TYPE_DICT_ENTRY,
+								NULL, &entry);
+
+	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
+
+	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
+			DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_STRING_AS_STRING,
+								&value);
+
+	dbus_message_iter_open_container(&value, DBUS_TYPE_ARRAY,
+					DBUS_TYPE_STRING_AS_STRING, &iter);
+	__connman_notifier_list_enabled(&iter);
+	dbus_message_iter_close_container(&value, &iter);
+
+	dbus_message_iter_close_container(&entry, &value);
+
+	dbus_message_iter_close_container(dict, &entry);
+}
+
+static void append_connected_technologies(DBusMessageIter *dict)
+{
+	DBusMessageIter entry, value, iter;
+	const char *key = "ConnectedTechnologies";
+
+	dbus_message_iter_open_container(dict, DBUS_TYPE_DICT_ENTRY,
+								NULL, &entry);
+
+	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
+
+	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
+			DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_STRING_AS_STRING,
+								&value);
+
+	dbus_message_iter_open_container(&value, DBUS_TYPE_ARRAY,
+					DBUS_TYPE_STRING_AS_STRING, &iter);
+	__connman_notifier_list_connected(&iter);
 	dbus_message_iter_close_container(&value, &iter);
 
 	dbus_message_iter_close_container(&entry, &value);
@@ -200,8 +248,14 @@ static DBusMessage *get_properties(DBusConnection *conn,
 	connman_dbus_dict_append_variant(&dict, "OfflineMode",
 				DBUS_TYPE_BOOLEAN, &global_offlinemode);
 
-	append_technologies(FALSE, &dict);
-	append_technologies(TRUE, &dict);
+	append_available_technologies(&dict);
+	append_enabled_technologies(&dict);
+	append_connected_technologies(&dict);
+
+	str = __connman_service_default();
+	if (str != NULL)
+		connman_dbus_dict_append_variant(&dict, "DefaultTechnology",
+						DBUS_TYPE_STRING, &str);
 
 	dbus_message_iter_close_container(&array, &dict);
 
@@ -237,6 +291,8 @@ static DBusMessage *set_property(DBusConnection *conn,
 
 		global_offlinemode = offlinemode;
 
+		__connman_storage_save_global();
+
 		__connman_device_set_offlinemode(offlinemode);
 	} else if (g_str_equal(name, "ActiveProfile") == TRUE) {
 		const char *str;
@@ -250,6 +306,26 @@ static DBusMessage *set_property(DBusConnection *conn,
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
 
+static DBusMessage *get_state(DBusConnection *conn,
+					DBusMessage *msg, void *data)
+{
+	const char *str;
+
+	DBG("conn %p", conn);
+
+	if (__connman_security_check_privilege(msg,
+					CONNMAN_SECURITY_PRIVILEGE_PUBLIC) < 0)
+		return __connman_error_permission_denied(msg);
+
+	if (__connman_element_count(NULL, CONNMAN_ELEMENT_TYPE_CONNECTION) > 0)
+		str = "online";
+	else
+		str = "offline";
+
+	return g_dbus_create_reply(msg, DBUS_TYPE_STRING, &str,
+						DBUS_TYPE_INVALID);
+}
+
 static DBusMessage *add_profile(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
@@ -279,7 +355,7 @@ static DBusMessage *remove_profile(DBusConnection *conn,
 static DBusMessage *request_scan(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
-	enum connman_device_type type;
+	enum connman_service_type type;
 	const char *str;
 	int err;
 
@@ -289,11 +365,11 @@ static DBusMessage *request_scan(DBusConnection *conn,
 							DBUS_TYPE_INVALID);
 
 	if (g_strcmp0(str, "") == 0)
-		type = CONNMAN_DEVICE_TYPE_UNKNOWN;
+		type = CONNMAN_SERVICE_TYPE_UNKNOWN;
 	else if (g_strcmp0(str, "wifi") == 0)
-		type = CONNMAN_DEVICE_TYPE_WIFI;
+		type = CONNMAN_SERVICE_TYPE_WIFI;
 	else if (g_strcmp0(str, "wimax") == 0)
-		type = CONNMAN_DEVICE_TYPE_WIMAX;
+		type = CONNMAN_SERVICE_TYPE_WIMAX;
 	else
 		return __connman_error_invalid_arguments(msg);
 
@@ -313,7 +389,7 @@ static DBusMessage *request_scan(DBusConnection *conn,
 static DBusMessage *enable_technology(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
-	enum connman_device_type type;
+	enum connman_service_type type;
 	const char *str;
 	int err;
 
@@ -323,15 +399,15 @@ static DBusMessage *enable_technology(DBusConnection *conn,
 							DBUS_TYPE_INVALID);
 
 	if (g_strcmp0(str, "ethernet") == 0)
-		type = CONNMAN_DEVICE_TYPE_ETHERNET;
+		type = CONNMAN_SERVICE_TYPE_ETHERNET;
 	else if (g_strcmp0(str, "wifi") == 0)
-		type = CONNMAN_DEVICE_TYPE_WIFI;
+		type = CONNMAN_SERVICE_TYPE_WIFI;
 	else if (g_strcmp0(str, "wimax") == 0)
-		type = CONNMAN_DEVICE_TYPE_WIMAX;
+		type = CONNMAN_SERVICE_TYPE_WIMAX;
 	else if (g_strcmp0(str, "bluetooth") == 0)
-		type = CONNMAN_DEVICE_TYPE_BLUETOOTH;
-	else if (g_strcmp0(str, "gps") == 0)
-		type = CONNMAN_DEVICE_TYPE_GPS;
+		type = CONNMAN_SERVICE_TYPE_BLUETOOTH;
+	else if (g_strcmp0(str, "cellular") == 0)
+		type = CONNMAN_SERVICE_TYPE_CELLULAR;
 	else
 		return __connman_error_invalid_arguments(msg);
 
@@ -351,7 +427,7 @@ static DBusMessage *enable_technology(DBusConnection *conn,
 static DBusMessage *disable_technology(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
-	enum connman_device_type type;
+	enum connman_service_type type;
 	const char *str;
 	int err;
 
@@ -361,15 +437,15 @@ static DBusMessage *disable_technology(DBusConnection *conn,
 							DBUS_TYPE_INVALID);
 
 	if (g_strcmp0(str, "ethernet") == 0)
-		type = CONNMAN_DEVICE_TYPE_ETHERNET;
+		type = CONNMAN_SERVICE_TYPE_ETHERNET;
 	else if (g_strcmp0(str, "wifi") == 0)
-		type = CONNMAN_DEVICE_TYPE_WIFI;
+		type = CONNMAN_SERVICE_TYPE_WIFI;
 	else if (g_strcmp0(str, "wimax") == 0)
-		type = CONNMAN_DEVICE_TYPE_WIMAX;
+		type = CONNMAN_SERVICE_TYPE_WIMAX;
 	else if (g_strcmp0(str, "bluetooth") == 0)
-		type = CONNMAN_DEVICE_TYPE_BLUETOOTH;
-	else if (g_strcmp0(str, "gps") == 0)
-		type = CONNMAN_DEVICE_TYPE_GPS;
+		type = CONNMAN_SERVICE_TYPE_BLUETOOTH;
+	else if (g_strcmp0(str, "cellular") == 0)
+		type = CONNMAN_SERVICE_TYPE_CELLULAR;
 	else
 		return __connman_error_invalid_arguments(msg);
 
@@ -461,6 +537,7 @@ static DBusMessage *unregister_agent(DBusConnection *conn,
 static GDBusMethodTable manager_methods[] = {
 	{ "GetProperties",     "",      "a{sv}", get_properties     },
 	{ "SetProperty",       "sv",    "",      set_property       },
+	{ "GetState",          "",      "s",     get_state          },
 	{ "AddProfile",        "s",     "o",     add_profile        },
 	{ "RemoveProfile",     "o",     "",      remove_profile     },
 	{ "RequestScan",       "s",     "",      request_scan       },
@@ -531,7 +608,10 @@ static DBusMessage *nm_state(DBusConnection *conn,
 	if (reply == NULL)
 		return NULL;
 
-	state = NM_STATE_DISCONNECTED;
+	if (__connman_element_count(NULL, CONNMAN_ELEMENT_TYPE_CONNECTION) > 0)
+		state = NM_STATE_CONNECTED;
+	else
+		state = NM_STATE_DISCONNECTED;
 
 	dbus_message_append_args(reply, DBUS_TYPE_UINT32, &state,
 							DBUS_TYPE_INVALID);
@@ -546,6 +626,57 @@ static GDBusMethodTable nm_methods[] = {
 	{ },
 };
 
+static int manager_load(void)
+{
+	GKeyFile *keyfile;
+	GError *error = NULL;
+	connman_bool_t offlinemode;
+
+	DBG("");
+
+	keyfile = __connman_storage_open();
+	if (keyfile == NULL)
+		return -EIO;
+
+	offlinemode = g_key_file_get_boolean(keyfile, "global",
+						"OfflineMode", &error);
+	if (error == NULL) {
+		global_offlinemode = offlinemode;
+
+		__connman_device_set_offlinemode(offlinemode);
+	}
+	g_clear_error(&error);
+
+	__connman_storage_close(keyfile, FALSE);
+
+	return 0;
+}
+
+static int manager_save(void)
+{
+	GKeyFile *keyfile;
+
+	DBG("");
+
+	keyfile = __connman_storage_open();
+	if (keyfile == NULL)
+		return -EIO;
+
+	g_key_file_set_boolean(keyfile, "global",
+					"OfflineMode", global_offlinemode);
+
+	__connman_storage_close(keyfile, TRUE);
+
+	return 0;
+}
+
+static struct connman_storage manager_storage = {
+	.name		= "manager",
+	.priority	= CONNMAN_STORAGE_PRIORITY_LOW,
+	.global_load	= manager_load,
+	.global_save	= manager_save,
+};
+
 static DBusConnection *connection = NULL;
 static gboolean nm_compat = FALSE;
 
@@ -557,6 +688,9 @@ int __connman_manager_init(DBusConnection *conn, gboolean compat)
 	if (connection == NULL)
 		return -1;
 
+	if (connman_storage_register(&manager_storage) < 0)
+		connman_error("Failed to register manager storage");
+
 	g_dbus_register_interface(connection, CONNMAN_MANAGER_PATH,
 					CONNMAN_MANAGER_INTERFACE,
 					manager_methods,
@@ -576,6 +710,8 @@ void __connman_manager_cleanup(void)
 {
 	DBG("conn %p", connection);
 
+	connman_storage_unregister(&manager_storage);
+
 	if (nm_compat == TRUE) {
 		g_dbus_unregister_interface(connection, NM_PATH, NM_INTERFACE);
 	}
